#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________


import coopr.opt
from coopr.pyomo.base import expr, Var, Constraint, Objective
from coopr.pyomo.base.var import _VarValue
from coopr.pyomo.base.param import _ParamValue
from coopr.pyomo.base.expr import *
from coopr.pyomo.base.numvalue import *

class ProblemWriterUtils(coopr.opt.AbstractProblemWriter):
    """
    Class that contains utilities for organizing model data before
    the problem is written.  

    WEH: Perhaps these should be moved into AbstractProblemWriter.  
    We'll see how this code evolves...
    
    """

    def __init__(self, problem_format):
        coopr.opt.AbstractProblemWriter.__init__(self,problem_format)
        self._varmap={}

    def _name_fix(self, name):
         name = name.replace("\"","")
         name = name.replace("%","_")
         name = name.replace("[","_")
         name = name.replace("]","_")
         name = name.replace("(","_")
         name = name.replace(")","_")
         name = name.replace(" ","")
         name = name.replace(",","_")
         name = name.replace("-","_")
         name = name.replace("'","")
         name = name.replace("`","")
         return name

    #
    # Identify variables and confirm that the expression is linear
    #
    def _Collect1(self,exp):
        #
        # Expression
        #
        if isinstance(exp,expr.Expression):
            #
            # SumExpression
            #
            if isinstance(exp,expr._SumExpression):
                for i in xrange(len(exp._args)):
                    self._Collect1(exp._args[i])
            #
            # Identity
            #
            elif isinstance(exp,expr._IdentityExpression):
                self._Collect1(exp._args[0])
            #
            # Product
            #
            elif isinstance(exp,expr._ProductExpression):
                v = "0"
                for i in xrange(len(exp._args)):
                    e = exp._args[i]
                    if isinstance(e,Var):
                        if v != "0":
                           raise ValueError, "Two variables in ProductExpression:",e
                        e._varval[None]._sno = 0
                    elif isinstance(e,_VarValue):
                        if v != "0":
                           raise ValueError, "Two variables in ProductExpression:",e
                        e._sno = 0
                    elif not isinstance(e,NumericConstant) and not isinstance(e,_ParamValue):
                        print "ERROR: Unexpected item of type=" + e.__class__.__name__ + " encountered in product expression during simplification (_Collect1)"
                        print "Parent expression: "
                        exp.pprint()
                        print "Offending item: "
                        e.pprint()
                        raise ValueError, "Unexpected item in ProductExpression - failed to simplify expression"
                
            elif isinstance(exp,expr._MinusExpression):
                self._Collect1(exp._args[0])
                self._Collect1(exp._args[1])
            #
            # ERROR
            #
            else:
                raise ValueError, "Unsupported expression type: "+str(type(exp))
        #
        # Variable Value
        #
        elif isinstance(exp,_VarValue):
           exp._sno = 0
           
        # "pure" (non-indexed) variables must be handled a bit differently,
        # forcing the "_sno" on the variable value explicitly.
        elif isinstance(exp,Var):
           exp._varval[None]._sno = 0          
        #
        # If not a constant, then this is an error
        #
        elif not isinstance(exp,NumericConstant) and not isinstance(exp,_ParamValue):
            #print "here",exp.value,dir(exp),exp._name,type(exp.value)
            raise ValueError, "ERROR: Unexpected expression type in _Collect1: " + str(exp)


    #
    # Identify variables and confirm that the expression is linear
    #
    def _Collect2(self, exp, x, scale=1.0):
        #
        # Expression
        #
        if isinstance(exp,expr.Expression):
                #
                # Sum
                #
                if isinstance(exp,expr._SumExpression):
                   for i in xrange(len(exp._args)):
                     x = self._Collect2(exp._args[i], x, scale)
                #
                # Identity
                #
                elif isinstance(exp,expr._IdentityExpression):
                   x = self._Collect2(exp._args[0], x, scale)
                #
                # Product
                #
                elif isinstance(exp,expr._ProductExpression):
                   c = scale
                   ve = v = "0"
                   for i in xrange(len(exp._args)):
                     e = exp._args[i]
                     if isinstance(e,NumericConstant) or isinstance(e,_ParamValue):
                        c *= e.value
                     elif isinstance(e,Var):
                        if v != "0":
                           raise ValueError, "ERROR: two variables in ProductExpression:",e
                        v = self._name_fix(e._varval[None].name)
                        ve = e._varval[None]
                     elif isinstance(e,_VarValue):
                        if v != "0":
                           raise ValueError, "ERROR: two variables in ProductExpression:",e
                        v = self._name_fix(e.name)
                        ve = e
                     else:
                        raise ValueError, "ERROR: unexpected item in ProductExpression:"+str(e)
                   if x.has_key(v):
                      xv = x[v]
                      x[v] = (xv[0]+c,xv[1])
                   else:
                      x[v] = (c,ve)
                #
                # Minus
                #
                elif isinstance(exp,expr._MinusExpression):
                    self._Collect2(exp._args[0], x, scale)
                    self._Collect2(exp._args[1], x, -scale)
                #
                # ERROR
                #
                else:
                        raise ValueError, "Unsupported expression type: "+str(exp)
        #
        # Constant
        #
        elif isinstance(exp,NumericConstant) or isinstance(exp, _ParamValue):
           c = exp.value * scale
           if x.has_key("0"):
              xv = x["0"]
              x["0"] = (xv[0]+c,xv[1])
           else:
              x["0"] = (c,"0")
        #
        # Variable
        #
        elif isinstance(exp,_VarValue) or isinstance(exp,Var):
          v = self._name_fix(exp.name)
          if x.has_key(v):
               xv = x[v]
               x[v] = (xv[0] + scale, xv[1])
          else:
               x[v] = (scale, exp)
        #
        # ERROR
        #
        else:
           raise ValueError, "Unexpected expression type in _Collect2:"+str(exp)
        return x


    def _Collect3(self, exp):
        x = self._Collect2(exp,{})
        y = {}
        for i in x:
           if x[i][0] != 0.:
              y[i] = x[i]
        return y


    def _Collect(self,model):
         Vars = model.active_components(Var)
         Con = model.active_components(Constraint)
         Obj = model.active_components(Objective)
         Con1 = []
         #
         # Indicate that all variables are unused
         #
         for var in Vars.values():
           for V in var._varval.keys():
             var._varval[V]._sno = -1
         #
         # Call _Collect1 to find the variables that are used in 
         # the objective and constraints
         #
         for key in Obj.keys():
           for ondx in Obj[key]._expr:
             try:
                self._Collect1(Obj[key]._expr[ondx])
             except ValueError, str:
                print "ERROR: Detected issue during simplification of objective (method=ProblemWriterUtils::_Collect1)"
                print ("ISSUE: " + `str`),
                if ondx == None:
                   print ""
                else:
                   print ", Index="+`ondx`
                print "Expression=",
                print Obj[key]._expr[ondx].pprint()
                raise ValueError,""
         for key in Con.keys():
           C = Con[key]
           for cndx in C.keys():
              try:
                 self._Collect1(C._body[cndx])
              except ValueError, str:
                 print "ERROR: Detected issue during simplification of constraint="+C._name,
                 if cndx == None:
                    print ""
                 else:
                    print ", Index="+`cndx`
                 print "Expression=",
                 print C._body[cndx].pprint()
                 raise ValueError,""
         #
         # Count the number of variables, and order them
         #
         sno = 0
         for var in Vars.values():
           Vv = var._varval
           for V in Vv:
             if Vv[V]._sno != -1:
                Vv[V]._sno = sno
                self._varmap[self._name_fix(Vv[V].name)] = Vv[V]
                sno += 1
         model.nsno = sno
         #
         # Collect the linear terms
         #
         for key in Obj.keys():
           Obj[key]._linterm = {}
           for ondx in Obj[key]._expr:
             Obj[key]._linterm[ondx] = self._Collect3(Obj[key]._expr[ondx])
         for key in Con.keys():
           C = Con[key]
           C._linterm = {}
           Cnz = []
           nt = 0
           for cndx in C.keys():
             t = C._linterm[cndx] = self._Collect3(C._body[cndx])
             lt = len(t)
             if lt > 0:
                Cnz.append(cndx)
                nt += 1
           if nt > 0:
              Con1.append(key)
           C._Cnz = Cnz
         model.Cnontriv = Con1




#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________

import pyutilib.component.core
from coopr.pyomo.base import IModelTransformation
from coopr.pyomo.base import Var
from coopr.pyomo import BooleanSet, IntegerSet, RangeSet, Reals


class RelaxIntegrality(pyutilib.component.core.SingletonPlugin):
    """
    This plugin relaxes integrality in a Pyomo model.
    """

    pyutilib.component.core.implements(IModelTransformation)

    def __init__(self, **kwds):
        kwds['name'] = "relax_integrality"
        pyutilib.component.core.Plugin.__init__(self, **kwds)

    def apply(self, model, **kwds):
        #
        # Clone the model
        #
        M = model.clone()
        #
        # Iterate over all variables, replacing the domain with a real-valued domain
        # and setting appropriate bounds.
        #
        # Note: this technique is not extensible.  If a user creates a new integer variable
        # class then we'd need to add it here (unless it was a subset of one of the sets we
        # detect here.  Alternatively, we could make the variables responsible for managing 
        # the generation of a relaxed variable.  But that would pollute the variable objects
        # with logic about generating a specific relaxation.  That's probably a worse alternative.
        #
        comp = M.components[Var]
        for var in comp.values():
            if isinstance(var.domain, BooleanSet):
                var.domain=Reals
                var._bounds = (0.0,1.0)
            elif isinstance(var.domain, IntegerSet):
                var.domain=Reals
                var._bounds = var.domain._bounds
            elif isinstance(var.domain, RangeSet):
                var.domain=Reals
                var._bounds = var.domain._bounds
        M.statistics.number_of_binary_variables = 0
        M.statistics.number_of_integer_variables = 0
        M.statistics.number_of_continuous_variables = len(M._var)
        return M




# an example of a piecewise function constructed from breakpoint slopes and rules. 
# minimum is at (0,0), where the objective function value is 0.

from coopr.pyomo import *

def compute_breakpoints(m):
  return [0, 2.5, 5]

def compute_slopes(m):
  return [-1, 1, 2, 3]

model = AbstractModel()

model.X = Var()

model.Z = Piecewise(breakpoint_rule=compute_breakpoints,
                    slope_rule=compute_slopes,
                    value=model.X)

model.obj = Objective(rule = lambda model: model.Z, sense=minimize)



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

#
# Problem Writer for CPLEX LP Format Files
#

import math

from coopr.opt import ProblemFormat
from coopr.opt.base import AbstractProblemWriter
from coopr.pyomo.base import BooleanSet, Constraint, expr, IntegerSet
from coopr.pyomo.base import Objective, NumericConstant, SOSConstraint, Var
from coopr.pyomo.base import VarStatus, value
from coopr.pyomo.base.numtypes import minimize, maximize
from coopr.pyomo.expr import is_constant, is_nonlinear, is_quadratic

from pyutilib.component.core import alias
from pyutilib.misc import deprecated


def convert_name(namestr):
    
    return namestr.replace('[','(').replace(']',')')

class ProblemWriter_cpxlp(AbstractProblemWriter):

    alias('cpxlp')
    alias('lp')

    def __init__(self):
        
        AbstractProblemWriter.__init__(self,ProblemFormat.cpxlp)

        # temporarily placing attributes used in extensive-form writing
        # here, for eventual migration to the base class.
        self._output_objectives = True
        self._output_constraints = True
        self._output_variables = True
            # means we're outputting *some* variables - types determined by
            # following flags.

        # building on the above, partition out the variables that should
        # be written if the _output_variables attribute is set to True.
        # this is useful when writing components of multiple models to a
        # single LP file. unfortunately, the CPLEX LP format requires all
        # integer and binary (and continuous) variables to appear in a
        # single continuous block.
        #
        # SOS constraints must come after Bounds, General, Binary, and
        # Semi-Continuous sections
        #
        self._output_continuous_variables = True
        self._output_integer_variables = True
        self._output_binary_variables = True

        # do I pre-pend the model name to all identifiers I output?
        # useful in cases where you are writing components of multiple
        # models to a single output LP file.
        self._output_prefixes = False


    def __call__(self, model, filename):
        
        if filename is None:
           filename = model.name + ".lp"
        OUTPUT=open(filename,"w")
        symbol_map = self._print_model_LP(model,OUTPUT)
        OUTPUT.close()
        return filename, symbol_map


    def _get_bound(self, exp):
        
        if isinstance(exp,expr._IdentityExpression):
           return self._get_bound(exp._args[0])
        elif exp.is_constant():
           return exp()
        else:
           raise ValueError, "ERROR: nonconstant bound: " + str(exp)
           return None


    @staticmethod
    def _collect_key ( a ):
        
        return a[0]


    @staticmethod
    def _no_label_error ( var ):
        
        msg  = "Unable to find label for variable '%s'.\n"                    \
        'Possibly uninstantiated model.  Do any constraint or objective  '     \
        'rules reference the original model object, as opposed to the  '     \
        'passed model object? Alternatively, if you are developing code, '     \
        'has the model instance been pre-processed?'

        raise ValueError, msg % str(var)


    def _print_expr_linear(self, x, OUTPUT, **kwargs):
        
        """
        Return a expression as a string in LP format.

        Note that this function does not handle any differences in LP format
        interpretation by the solvers (e.g. CPlex vs GLPK).  That decision is
        left up to the caller.

        required arguments:
          x: A Pyomo linear encoding of an expression to write in LP format

        optional keyword arguments:
          is_objective: set True if printing a quadratic objective.  Required
             for CPlex LP quadratic handling differences between objectives
             and constraints. (boolean)

          print_offset: print expression offset [deprecated] (boolean)
        """

        is_objective = kwargs.pop('is_objective', False)
        print_offset = kwargs.pop('print_offset', False)

        constant_term = x[0]
        linear_terms = x[1]

        name_to_coefficient_map = {}

        for coefficient, var_value in linear_terms:

           if var_value.fixed is True:

              constant_term += (coefficient * var_value.value)

           else:

              prefix = ""
              if self._output_prefixes is True:
                 parent_var = var_value.var
                 prefix = convert_name(parent_var.model.name) + "_"

              if not var_value.label:
                 self._no_label_error(var_value)

              name = prefix + convert_name(var_value.label)

              # due to potential disabling of expression simplification,
              # variables might appear more than once - condense coefficients.
              name_to_coefficient_map[name] = coefficient + name_to_coefficient_map.get(name,0.0)

        sorted_names = sorted(name_to_coefficient_map.keys())

        for name in sorted_names:
     
           coefficient = name_to_coefficient_map[name]

           sign = '+'
           if coefficient < 0: sign = '-'
           print >>OUTPUT, '%s%f %s' % (sign, math.fabs(coefficient), name)

        if print_offset and (constant_term != 0.0):
            sign = '+'
            if constant_term < 0: sign = '-'
            print >>OUTPUT, '%s%f %s' % (sign, math.fabs(constant_term), 'ONE_VAR_CONSTANT')

        return constant_term


    def _print_expr_canonical(self, x, OUTPUT, **kwargs):
        
        """
        Return a expression as a string in LP format.

        Note that this function does not handle any differences in LP format
        interpretation by the solvers (e.g. CPlex vs GLPK).  That decision is
        left up to the caller.

        required arguments:
          x: A Pyomo canonical expression to write in LP format

        optional keyword arguments:
          is_objective: set True if printing a quadratic objective.  Required
             for CPlex LP quadratic handling differences between objectives
             and constraints. (boolean)

          print_offset: print expression offset [deprecated] (boolean)
        """

        is_objective = kwargs.pop('is_objective', False)
        print_offset = kwargs.pop('print_offset', False)

        #
        # Linear
        #
        if 1 in x:

           var_hashes = x[1].keys()

           name_to_coefficient_map = {}

           for this_hash in var_hashes:

              var_value = x[-1][this_hash.keys()[0]]
              coefficient = x[1][this_hash]

              prefix = ""

              if self._output_prefixes is True:
                 parent_var = var_value.var
                 prefix = convert_name(parent_var.model.name) + "_"

              label = var_value.label
              if not label:
                 self._no_label_error(var_value)
  
              name = prefix + convert_name(label)              

              name_to_coefficient_map[name] = coefficient

           sorted_names = sorted(name_to_coefficient_map.keys())
   
           for name in sorted_names:           
              coef = name_to_coefficient_map[name]
              sign = '+'
              if coef < 0: sign = '-'
              print >>OUTPUT, '%s%f %s' % (sign, math.fabs(coef), name)

        #
        # Quadratic
        #
        if 2 in x:

            print >>OUTPUT, "+ ["

            for id in sorted(x[2].keys()):

                coefficient = x[2][id]
                sign = '+'
                if coefficient < 0: 
                   sign = '-'
                   coefficient = math.fabs(coefficient)

                if is_objective:
                    coefficient *= 2
                # times 2 because LP format requires /2 for all the quadratic
                # terms /of the objective only/.  Discovered the last bit thru
                # trial and error.  Obnoxious.
                # Ref: ILog CPlex 8.0 User's Manual, p197.

                print >>OUTPUT, sign, coefficient,

                term_variables = []
                
                for var in id:

                    var_value = x[-1][var]
                    
                    label = var_value.label
                    if not label:
                        self._no_label_error(var_value)

                    prefix = ""

                    if self._output_prefixes is True:
                       parent_var = var_value.var
                       prefix = convert_name(parent_var.model.name) + "_"                        

                    name = prefix + convert_name(label) 
                    term_variables.append(name)

                if len(term_variables) == 2:
                   print >>OUTPUT, term_variables[0],"*",term_variables[1],
                else:
                   print >>OUTPUT, term_variables[0],"^ 2",
                
            print >>OUTPUT, ""

            print >>OUTPUT, "]",
            if is_objective:
                print >>OUTPUT, ' / 2'
                # divide by 2 because LP format requires /2 for all the quadratic
                # terms.  Weird.  Ref: ILog CPlex 8.0 User's Manual, p197
            else:
               print >>OUTPUT, ""


        #
        # Constant offset
        #
        offset=0.0
        if 0 in x:
            offset = x[0][None]
        if print_offset and offset != 0.0:
            sign = '+'
            if offset < 0: sign = '-'
            print >>OUTPUT, '%s%f %s' % (sign, math.fabs(offset), 'ONE_VAR_CONSTANT')

        #
        # Return constant offset
        #
        return offset


#    @deprecated
    def _print_quadterm(self, x, is_minimizing, OUTPUT):
        
        # The LP format doesn't allow for expression of constant terms in the
        # objective.  A work-around involves tracking the sum of constant terms
        # in the quadratic quadratic terms, and then writing that out with a
        # dummy variable forced equal to one.
        print >>OUTPUT, ""
        for arg in x._args:
            if isinstance(arg,expr._ProductExpression):
                # NOTE: We need to handle quadratics defined with the 'pow'
                # term here.
                # WARNING: The following code is very specific to progressive
                # hedging, with a very specific format assumed.  We do need to
                # handle more general expressions, but we'll worry about that
                # at a latter time.
                blend = arg._numerator[0]()

                if blend is 1:

                   rho = arg._numerator[1]()

                   pow_expression = arg._numerator[2]

                   base = pow_expression._args[0]
                   exponent = pow_expression._args[1]

                   if not isinstance(base,expr._SumExpression):
                       msg = 'Quadratic term base must be a _SumExpression'
                       raise ValueError, msg

                   if not isinstance(exponent,NumericConstant):
                       msg = 'Quadratic term exponent must be a NumericConstant'
                       raise ValueError, msg

                   variable = base._args[0]
                   offset = base._args[1]
                   if variable.status is not VarStatus.unused:

                      sign = '-'
                      if is_minimizing is True:
                          sign = '+'
                      print >>OUTPUT, '%s [ %s %s^2 ] / 2' % (
                        sign,
                        str(rho),
                        convert_name(variable.label)
                      )

                      sign = '-'
                      if (is_minimizing is True) == (offset.value <  0):
                          sign = '+'
                      print >>OUTPUT, '%s %s %s' % (
                         sign,
                         str(abs(rho*offset.value)),
                         convert_name( variable.label )
                      )

                      objective_offset = (rho * offset.value*offset.value /2.0)
                      fmt = ' -%s ONE_VAR_CONSTANT'
                      if is_minimizing is True: fmt = ' +%s ONE_VAR_CONSTANT'
                      print >>OUTPUT, fmt % str(objective_offset)

            elif isinstance(arg,NumericConstant):
                # this is the "0.0" element that forms the initial expression
                # -- the quadratic sub-expressions aren't known to the presolve
                # routines. ideally unnecessary - hacked in for now.
                pass

            else:
                msg = '%s\nUnknown expression sub-type found in quadratic '   \
                      'objective expression'
                raise ValueError, msg % `arg`


    @staticmethod
    def printSOS(con, name, OUTPUT, index=None):
        
        """
        Returns the SOS constraint (as a string) associated with con.
        If specified, index is passed to con.sos_set().

        Arguments:
        con    The SOS constraint object
        name   The name of the variable
        OUTPUT The output stream
        index  [Optional] the index to pass to the sets indexing the variables.
        """

        # The name of the variable being indexed
        varName = str(con.sos_vars())

        # The list of variable names to be printed, including indices
        varNames = []

        # Get all the variables
        if index is None:
            tmpSet = con.sos_set()
        else:
            tmpSet = con.sos_set()[index]
        for x in tmpSet:
            strX = str(x)
            if strX[0] == "(":
                # its a tuple, remove whitespace
                varNames.append(varName + strX.replace(" ",""))
            else:
                # its a single number, add parenthesis
                varNames.append(varName + "(" + strX + ")")

        conNameIndex = ""
        if index is not None:
            conNameIndex = str(index)

        print >>OUTPUT, '%s%s: S%s::' % (name, conNameIndex, con.sos_level())

        # We need to 'weight' each variable
        # For now we just increment a counter
        for i in range(0, len(varNames)):
            print >>OUTPUT, '%s:%f' % (varNames[i], i+1)

    def _print_model_LP(self, model, OUTPUT):

        symbol_map = {}
        _obj = model.active_components(Objective)

        supports_quadratic = model.has_capability('quadratic')

        #
        # Objective
        #
        if self._output_objectives is True:
            
           printed_quadterm = False
           if len(_obj) == 0:
               msg = "ERROR: No objectives defined for input model '%s'; "    \
                     ' cannot write legal LP file'
               raise ValueError, msg % str( model.name )

           if _obj[ _obj.keys()[0] ].sense == maximize:
              print >>OUTPUT, "max "
           else:
              print >>OUTPUT, "min "

           obj = _obj[ _obj.keys()[0] ]
           obj_keys = obj.keys()
           if len(obj_keys) > 1:
               keys = obj.name + ' (indexed by: %s)' % ', '.join(map(str, obj_keys))

               msg = "More than one objective defined for input model '%s'; " \
                     'Cannot write legal LP file\n'                           \
                     'Objectives: %s'
               raise ValueError, msg % ( str(model.name), keys )

           for key in obj_keys:

                if obj[key].repn is None:
                   raise RuntimeError, "No canonical representation identified for objective="+obj[key].name

                if is_constant(obj[key].repn):

                    print ("Warning: Constant objective detected, replacing " +
                           "with a placeholder to prevent solver failure.")
                    
                    print >>OUTPUT, obj._data[None].label+": +0.0 ONE_VAR_CONSTANT"

                    # Skip the remaining logic of the section
                    continue

                if is_quadratic( obj[key].repn ):
                    if not supports_quadratic:
                        msg  = 'Solver unable to handle quadratic '           \
                               "objective expressions.  Objective at issue: %s%s."
                        if key is None:
                           msg %= (obj.name, " ")
                        else:
                           msg %= (obj.name, '[%s]' % key )
                        raise ValueError, msg

                elif is_nonlinear( obj[key].repn ):
                    msg  = "Cannot write legal LP file.  Objective '%s%s' "  \
                           'has nonlinear terms that are not quadratic.'
                    if key is None: msg %= (obj.name, '')
                    else:           msg %= (obj.name, '[%s]' % key )
                    raise ValueError, msg

                symbol_map['__default_objective__'] = obj._data[None].label
                print >>OUTPUT, obj._data[None].label+':'

                offset = self._print_expr_canonical(obj[key].repn,
                                                    OUTPUT,
                                                    print_offset=True,
                                                    is_objective=True)

           if obj._quad_subexpr is not None:
               self._print_quadterm(obj._quad_subexpr, (_obj[ _obj.keys()[0] ].sense == minimize), OUTPUT)
               printed_quadterm = True
               
           print >>OUTPUT, ""

        # Constraints
        #
        # If there are no non-trivial constraints, you'll end up with an empty
        # constraint block. CPLEX is OK with this, but GLPK isn't. And
        # eliminating the constraint block (i.e., the "s.t." line) causes GLPK
        # to whine elsewhere. output a warning if the constraint block is empty,
        # so users can quickly determine the cause of the solve failure.

        if self._output_constraints is True:

           # for now, if this routine isn't writing everything, then assume a
           # meta-level handler is dealing with the writing of the transitional
           # elements - these should probably be done in the form of
           # "end_objective()" and "end_constraint()" helper methods.
           if self._output_objectives == self._output_variables == True:
               print >>OUTPUT, "s.t."
               print >>OUTPUT, ""

           active_constraints = model.active_components(Constraint)
           have_nontrivial = False
           for constraint in active_constraints.values():
             if constraint.trivial:
                continue

             have_nontrivial=True
             
             for index in sorted(constraint.keys()):

               constraint_data = constraint[index]
               if not constraint_data.active:
                    continue

               # if expression trees have been linearized, then the canonical
               # representation attribute on the constraint data object will
               # be equal to None.
               if (constraint_data.repn is not None):

                  # There are conditions, e.g., when fixing variables, under which
                  # a constraint block might be empty.  Ignore these, for both
                  # practical reasons and the fact that the CPLEX LP format
                  # requires a variable in the constraint body.  It is also
                  # possible that the body of the constraint consists of only a
                  # constant, in which case the "variable" of
                  if is_constant(constraint_data.repn):
                      # this happens *all* the time in many applications,
                      # including PH - so suppress the warning.
                      #
                      #msg = 'WARNING: ignoring constraint %s[%s] which is ' \
                      #      'constant'
                      #print msg % (str(C),str(index))
                      continue
   
                  if is_quadratic( constraint_data.repn ):
                      if not supports_quadratic:
                          msg  = 'Solver unable to handle quadratic expressions.'\
                                 "  Constraint at issue: '%s%%s'"
                          msg %= constraint.name
                          if index is None: msg %= ''
                          else: msg %= '[%s]' % index
   
                          raise ValueError, msg

                  elif is_nonlinear( constraint_data.repn ):
                      msg = "Cannot write legal LP file.  Constraint '%s%%s' "   \
                            'has a body with nonlinear terms.'
                      if index is None: msg %= ( constraint.name, '')
                      else:            msg %= ( constraint.name, '[%s]' % index )
   
                      raise ValueError, msg

               prefix = ""
               if self._output_prefixes is True:
                   if constraint.model is None:
                      msg = "Constraint '%s' has no model attribute - no "   \
                            'label prefix can be assigned'
                      raise RuntimeError, msg % constraint_data.label
                   prefix = constraint.model.name+"_"

               con_name = convert_name(constraint_data.label)

               if constraint_data._equality:
                  label = '%sc_e_%s_' % (prefix, con_name)
                  symbol_map['%sc_e_%s_' % (prefix, constraint_data.label)] = constraint_data.label
                  print >>OUTPUT, label+':'
                  if constraint_data.lin_body is not None:
                     offset = self._print_expr_linear(constraint_data.lin_body, OUTPUT)               
                  else:
                     offset = self._print_expr_canonical(constraint_data.repn, OUTPUT)               
                  bound = constraint_data.lower
                  bound = str(self._get_bound(bound) - offset)
                  print >>OUTPUT, "=", bound
                  print >>OUTPUT, ""
               else:
                  # TBD: ENCAPSULATE THE IF-ELSE INTO A SINGLE UTILITY METHOD
                  # TBD: MAKE THE _data and C[ndx] calls consistent - everything should just reference the data directly
                  if constraint_data.lower is not None:
                     label = '%sc_l_%s_' % (prefix, con_name)
                     symbol_map['%sc_l_%s_' % (prefix, constraint_data.label)] = constraint_data.label
                     print >>OUTPUT, label+':'
                     if constraint_data.lin_body is not None:
                        offset = self._print_expr_linear(constraint_data.lin_body, OUTPUT)                                    
                     else:
                        offset = self._print_expr_canonical(constraint_data.repn, OUTPUT)                                    
                     bound = constraint_data.lower
                     bound = str(self._get_bound(bound) - offset)
                     print >>OUTPUT, ">=", bound
                     print >>OUTPUT, ""                     
                  if constraint_data.upper is not None:
                     label = '%sc_u_%s_' % (prefix, con_name)
                     symbol_map['%sc_u_%s_' % (prefix, constraint_data.label)] = constraint_data.label
                     print >>OUTPUT, label+':'
                     if constraint_data.lin_body is not None:
                        offset = self._print_expr_linear(constraint_data.lin_body, OUTPUT)                                                         
                     else:
                        offset = self._print_expr_canonical(constraint_data.repn, OUTPUT)                                                         
                     bound = constraint_data.upper
                     bound = str(self._get_bound(bound) - offset)
                     print >>OUTPUT, "<=", bound
                     print >>OUTPUT, ""                                          

           if not have_nontrivial:
               print 'WARNING: Empty constraint block written in LP format '  \
                     '- solver may error'

           # the CPLEX LP format doesn't allow constants in the objective (or
           # constraint body), which is a bit silly.  To avoid painful
           # book-keeping, we introduce the following "variable", constrained
           # to the value 1.  This is used when quadratic terms are present.
           # worst-case, if not used, is that CPLEX easily pre-processes it out.
           prefix = ""
           if self._output_prefixes is True:
              prefix = model.name + "_"
           print >>OUTPUT, '%sc_e_ONE_VAR_CONSTANT: ' % prefix
           print >>OUTPUT, '%sONE_VAR_CONSTANT = 1.0' % prefix
           print >>OUTPUT, ""

        #
        # Bounds
        #

        if self._output_variables is True:

           # For now, if this routine isn't writing everything, then assume a
           # meta-level handler is dealing with the writing of the transitional
           # elements - these should probably be done in the form of
           # "end_objective()" and "end_constraint()" helper methods.
           if True == self._output_objectives == self._output_constraints:
               print >>OUTPUT, "bounds "

           # Scan all variables even if we're only writing a subset of them.
           # required because we don't store maps by variable type currently.

           # Track the number of integer and binary variables, so you can
           # output their status later.
           niv = nbv = 0

           VAR = model.active_components(Var)

           for variable_name in sorted(VAR.keys()):

               var = VAR[variable_name]

               for index in sorted(var.keys()):
                   v = var[index]
                   if isinstance(v.domain, IntegerSet):   niv += 1
                   elif isinstance(v.domain, BooleanSet): nbv += 1

               if self._output_continuous_variables is True:

                   for index in sorted(var._varval.keys()):

                       if not var._varval[index].active:
                           continue
                       prefix = ""
                       if self._output_prefixes is True:
                           prefix = convert_name(var.model.name)+"_"

                       # if the variable isn't referenced in the model, don't
                       # output bounds...
                       if var[index].id != -1:
                           # in the CPLEX LP file format, the default variable
                           # bounds are 0 and +inf.  These bounds are in
                           # conflict with Pyomo, which assumes -inf and inf
                           # (which we would argue is more rational).
                           print >>OUTPUT,"   ",
                           if var[index].lb is not None:
                             print >>OUTPUT, str(value(var[index].lb())), "<= ",
                           else:
                             print >>OUTPUT, " -inf <= ",
                           name_to_output = prefix+convert_name(var[index].label)
                           if name_to_output == "e":
                               msg = 'Attempting to write variable with name' \
                                     "'e' in a CPLEX LP formatted file - "    \
                                     'will cause a parse failure due to '     \
                                     'confusion with numeric values '         \
                                     'expressed in scientific notation'
                               raise ValueError, msg
                           print >>OUTPUT, name_to_output,
                           if var[index].ub is not None:
                               print >>OUTPUT, " <=", str(value(var[index].ub()))
                           else:
                               print >>OUTPUT, " <= +inf"

           if (niv > 0) and (self._output_integer_variables is True):

              # If we're outputting the whole model, then assume we can output
              # the "general" header. If not, then assume a meta-level process
              # is taking care of it.
              if True == self._output_objectives == self._output_constraints:
                 print >>OUTPUT, "general"

              prefix = ""
              if self._output_prefixes is True:
                 prefix = convert_name(var.model.name)+"_"

              for variable_name in sorted(VAR.keys()):
        
                var = VAR[variable_name]

                for index in sorted(var.integer_keys()):

                   if not var[index].active:
                      continue
                   if var[index].id != -1: # skip if variable not referenced
                       var_name = prefix+convert_name(var[index].label)
                       print >>OUTPUT, ' ', var_name

           if (nbv > 0) and (self._output_binary_variables is True):

              # If we're outputting the whole model, then assume we can output
              # the "binary" header. if not, then assume a meta-level process
              # is taking care of it.
              if True == self._output_objectives == self._output_constraints:
                 print >>OUTPUT, "binary"

              prefix = ""
              if self._output_prefixes is True:
                 prefix = convert_name(var.model.name)+"_"

              for variable_name in sorted(VAR.keys()):
        
                var = VAR[variable_name]

                for index in sorted(var.binary_keys()):
                   if not var[index].active:
                       continue
                   if var[index].id != -1: # skip if variable not referenced
                       var_name = prefix+convert_name(var[index].label)
                       print >>OUTPUT, ' ', var_name


        # SOS constraints
        #
        # For now, we write out SOS1 and SOS2 constraints in the cplex format
        #
        # All Component objects are stored in model._component, which is a
        # dictionary of {class: {objName: object}}.
        #
        # Consider the variable X,
        #
        #   model.X = Var(...)
        #
        # We print X to CPLEX format as X(i,j,k,...) where i, j, k, ... are the
        # indices of X.
        #
        # TODO: Allow users to specify the variables coefficients for custom
        # branching/set orders
        sosn = model.has_capability("sosn")
        sos1 = model.has_capability("sos1")
        sos2 = model.has_capability("sos2")

        if sosn or sos1 or sos2:
            writtenSOS = False
            constrs = model.components[SOSConstraint]
            for name in constrs:
                con = constrs[name]
                level = con.sos_level()
                if (level == 1 and sos1) or (level == 2 and sos2) or (sosn):
                    if writtenSOS == False:
                        print >>OUTPUT, "SOS"
                        writtenSOS = True
                    masterIndex = con.sos_set_set()
                    if None in masterIndex:
                        # A single constraint
                        self.printSOS(con, name, OUTPUT)
                    else:
                        # A series of indexed constraints
                        for index in masterIndex:
                            self.printSOS(con, name, OUTPUT, index)

        #
        # wrap-up
        #
        if self._output_objectives == self._output_constraints == True:
           #
           # End
           #
           print >>OUTPUT, "end "
        #
        return symbol_map



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['_ParamBase', 'Param']

import sys
import types
import logging

from component import *
from indexed_component import IndexedComponent
from numvalue import create_name, value
from plugin import ParamRepresentationFactory
from pyutilib.component.core import alias
from set_types import *
from sets import _BaseSet
from param_repn import _ParamValue

logger = logging.getLogger('coopr.pyomo')


class _ParamBase(IndexedComponent):
    """A parameter value, which may be defined over a index"""

    """ Constructor
        Arguments:
           name       The name of this parameter
           index      The index set that defines the distinct parameters.
                         By default, this is None, indicating that there
                         is a single parameter.
           within     A set that defines the type of values that
                         each parameter must be.
           validate   A rule for validating this parameter w.r.t. data
                         that exists in the model
           default    A set that defines default values for this parameter
           rule       A rule for setting up this parameter with existing model
                         data
           nochecking If true, various checks regarding valid domain and index
                      values are skipped. Only intended for use by algorithm
                      developers - not modelers!
    """
    def __init__(self, *args, **kwd):
        tkwd = {'ctype': kwd.pop("ctype", Param)}
        IndexedComponent.__init__(self, *args, **tkwd)

        self._initialize = kwd.pop('initialize', None )
        self._initialize = kwd.pop('rule', self._initialize )
        self._validate   = kwd.pop('validate', None )
        self.doc         = kwd.pop('doc', None )
        self.domain      = kwd.pop('within', Any )
        self.name        = kwd.pop('name', 'unknown')
        self.nochecking  = kwd.pop('nochecking',False)
        defaultval       = kwd.pop('default', None )

        self._default = _ParamValue(
            name   = self.name,
            domain = self.domain,
            value  = value(defaultval)
            )

        self.repn_type = kwd.pop('repn', 'pyomo_dict')
        self._repn        = ParamRepresentationFactory( self.repn_type, args=(self,) )
        if self._repn is None:
            raise ValueError, "Unknown parameter representation '%s'" % self.repn_type

        if ( kwd ):
            msg = "Unknown keywords to Param constructor: %s"
            raise ValueError, msg % repr(kwd)

    def initialize(self, data):
        self._initialize = data

    def pprint(self, ostream=None):
        if ostream is None:
           ostream = sys.stdout
        print >>ostream, "  ",self.name,":",
        print >>ostream, "\tSize="+str(len(self)),
        print >>ostream, "\tDomain="+self.domain.name
        if None in self.keys():
            if self._constructed is True:
               print >>ostream, "\t", self._repn(None)
            else:
               print >>ostream, "\t", "Not constructed"
        else:
            tmp=self._repn.keys(nondefault=True)
            tmp.sort()
            for key in tmp:
                val = self._repn(key)
                print >>ostream, "\t"+str(key)+" : "+str(val)
        if self._default.value is not None and not self._default.value is None:
           print >>ostream, "\tdefault: "+str(self._default.value)

    def keys(self):
        return self._repn.keys()

    def __contains__(self, element):
        return element in self._repn

    def __iter__(self):
        return self._repn.__iter__()

    def set_default(self, value):
        self._default.value = value
        self._repn.set_default(value)

    def as_numeric(self):
        if None in self._repn:
            return self._repn[None]
        return self

    def is_indexed(self):
        return self._ndim > 0

    def is_expression(self):
        return False

    def is_relational(self):
        return False

    def dim(self):
        return self._ndim

    def __len__(self):
        return len(self._repn)

    def __getitem__(self,ndx):
        """This method returns a _ParamVal object.  This object can be
           coerced to a numeric value using the value() function, or using
           explicity coercion with float().
        """
        return self._repn[ndx]

    def __setitem__(self,ndx,val):
        if self._index_set is not None:
            # this is a "hack", in that we still need to check for legal
            # sub-components in the input index.
            self._repn[ndx] = val
            return
        #
        if type(ndx) is tuple and len(ndx) == 1 and ndx[0] in self._index:
            ndx = ndx[0]
        if (None in self._index) and (ndx is not None):
            # allow for None indexing if this is truly a singleton
            msg = "Cannot set an array value in the simple parameter '%s'"
            raise KeyError, msg % self.name
        #
        if (self.nochecking is False) and (ndx not in self._index):
            msg = "Cannot set the value of array parameter '%s' with invalid " \
                  "index '%s'"
            raise KeyError, msg % ( self.name, str(ndx) )
        #
        self._repn[ndx] = val
        #
        # This is at the end, so the user can write their validation
        # tests as if the data was already added.
        #
        if (self.nochecking is False) and (not self._valid_indexed_value(val ,ndx, False)):

            msg = "Invalid parameter value: %s[%s] = '%s', value type=%s"
            raise ValueError, msg % ( self.name, str(ndx), str(val), str(type(val)) )

    def construct(self, data=None):
        """ Apply the rule to construct values in this set """
        if __debug__:
            if logger.isEnabledFor(logging.DEBUG):
                logger.debug("Constructing Param, name="+self.name+", from data="+`data`)
        if self._constructed:
            raise IOError, "Cannot reconstruct parameter '%s'" % self.name
            return
        self._constructed=True
        #
        # Code optimization with local variables
        #
        _name=self.name
        _domain=self.domain
        #
        self._repn.update_index()
        rule = getattr(self,'rule',None)
        if not rule is None:
            self._initialize=rule
        #
        # Construct using the initial data or the data loaded from an
        # external source.  Note:  data values will be queried in to following
        # order:
        #   1) the 'data' dictionary
        #   2) the self._initialize dictionary
        #   3) the default value
        #
        if data is not None or type(self._initialize) is not types.FunctionType:
            #
            # Singleton (non-indexed) parameter
            #
            #print self._repn, self._initialize, self._default.value, self._index
            if type(self._index) is dict:
                error = False
                if data is not None and None in data.keys():
                    self.value=data[None]
                elif not self._initialize is None:
                    if type(self._initialize) is not dict:
                        self.value=self._initialize
                    elif None in self._initialize:
                        self.value=self._initialize[None]
                    else:
                        error = True
                elif self._default.value is not None:
                    self.value=self._default.value
                else:
                    error = True
                if error:
                    msg = "Attempting to construct parameter '%s' without "   \
                          'parameter data'
                    raise ValueError, msg % _name

                #self.pprint()
                #print self.value#, self[None].value
                self[None] = self.value
                self._valid_indexed_value(self.value,(),True)
            else:
                #
                # Set external data values (if provided)
                #
                if data is not None:
                    #print 'z'
                    for key in data:
                        if type(key) is tuple and len(key)==1:
                            tmpkey=key[0]
                        else:
                            tmpkey=key
                        self.__setitem__(key, data[key])
                #
                # Initialize with initialization data.
                #
                elif self._initialize is not None:
                    #print 'y'
                    if type(self._initialize) is dict:
                        for key in self._initialize:
                            self.__setitem__(key, self._initialize[key])
                    else:
                        self._repn.set_item(val=self._initialize)
        #
        # Construct using the rule
        #
        elif type(self._initialize) is types.FunctionType:
            if (type(self._index) is dict) and (None in self._index):
                # singleton
                _tmp = []
                _tmp.append(self.model)
                tval = self._initialize(*_tmp)
                self.value = tval
                self[None] = self.value
            else:
                # non-singleton
                for val in self._index:
                    if isinstance(val, tuple):
                        _tmp = list(val)
                    elif val is None:
                        _tmp = []
                    else:
                        _tmp = [val]

                    if len(_tmp) > 0:
                        tname = create_name(_name,val)
                    else:
                         tname = _name

                    _tmp.append(self.model)
                    _tmp = tuple(_tmp)
                    tval = self._initialize(*_tmp)
                    self.__setitem__(val, tval)
        #
        if self.repn_type != 'pyomo_dict':
            self.model.__dict__[self.name] = self._repn.model_repn()

    def _valid_indexed_value(self, value, index, use_exception):
        if index is ():
            if None not in self._index:
                msg = "Invalid index '%s' for parameter '%s'"
                raise ValueError, msg % ( str(None), self.name )
        elif type(index) is tuple and len(index)==1:
            if index not in self._index and index[0] not in self._index:
                msg = "Invalid index '%s' for parameter '%s'"
                raise ValueError, msg % ( str(index[0]), self.name)
        elif index not in self._index:
            msg = "Invalid index '%s' for parameter '%s'"
            raise ValueError, msg % ( str(index), self.name)
        #
        if self._validate is not None:
           tmp = [value]
           if index == ():
              index = (None,)*self._ndim

           if index is not None:
              if type(index) is tuple:
                 tmp = tmp + list(index)
              else:
                 tmp = tmp + [ index ]

           tmp.append(self.model)
           tmp = tuple(tmp)
           if self._validate(*tmp):
              return True
        elif self.domain is Any or value in self.domain:
           return True
        #
        if use_exception:           #pragma:nocover
            msg = "Invalid value '%s' for parameter '%s'"
            raise ValueError, msg % ( str(value), self.name )
        #
        return False

    def data(self):
        return self._repn.data()


class _ParamElement(_ParamBase, _ParamValue):

    def __init__(self, *args, **kwds):

        repn_name = kwds.get('repn', 'pyomo_dict')

        _ParamBase.__init__(self, *args, **kwds)
        _ParamValue.__init__(self, name=kwds.get('name',None), domain=kwds.get('within',Any), value=kwds.get('default',None))        

        if repn_name == 'pyomo_dict':
            self._repn.set_item(None, self)
        else:
            self._repn.set_item(None, self._default.value)

    def __getstate__(self):
       result = _ParamValue.__getstate__(self)
       for key,value in self.__dict__.iteritems():
          result[key]=value
       return result    

    def __len__(self):
        return 1

    def keys(self):
        return [None]

    def __getitem__(self, key):
        if not key is None:
            raise ValueError, "Undefined key %s for parameter %s" % (key, self.name)
        return self

    def check_values(self):         #pragma:nocover
        #
        # Validate the values
        #
        if None not in self._index:
            raise ValueError, "Undefined value for parameter "+self.name
        if not self._valid_indexed_value(self.value,None,False):
            msg = "Parameter '%s' failed validation test.  Value: %s"
            raise ValueError, msg % ( self.name, str(self.value) )

    def __call__(self, exception=True):
        return self.value
    

class _ParamArray(_ParamBase):

    def __init__(self, *args, **kwds):
        _ParamBase.__init__(self, *args, **kwds)

    def __float__(self):
        msg = "Cannot access the value of array parameter '%s'"
        raise ValueError, msg % self.name

    def __int__(self):
        msg = "Cannot access the value of array parameter '%s'"
        raise ValueError, msg % self.name

    def set_value(self, value):
        for key in self._repn:
            self.__setitem__(key,value)

    def check_values(self):         #pragma:nocover
        #
        # Validate the values
        #
        for key in self._repn:
            if key not in self._index:
                msg = "Undefined value for parameter %s[%s]"
                raise ValueError, msg % ( self.name, str(key) )

            val = self._repn(key)
            if not self._valid_indexed_value( val, key, False ):
                msg = "Parameter %s[%s] failed validation test.  Value: %s"
                raise ValueError, msg % ( self.name, str(key), str(tval) )

    def reset(self):
        pass                        #pragma:nocover

    def display(self, ostream=None):
        self.pprint(ostream=ostream)

    def __str__(self):
        return str(self.name)


class Param(Component):
    """
    Data objects that are used to construct Pyomo models.
    """

    alias("Param", "Parameter data that is used to define a model instance.")

    def __new__(cls, *args, **kwds):
        if args == ():
            self = _ParamElement(*args, **kwds)
        else:
            self = _ParamArray(*args, **kwds)
        return self




#
# Unit Tests for Elements of a Model
#
# TestSimpleObj                Class for testing single objective
# TestArrayObj                Class for testing array of objective
#

import os
import sys
from os.path import abspath, dirname
sys.path.insert(0, dirname(dirname(abspath(__file__)))+os.sep+".."+os.sep+"..")
currdir = dirname(abspath(__file__))+os.sep

from coopr.pyomo.base import IntegerSet
from coopr.pyomo import *
from coopr.opt import *
from coopr.pyomo.base.var import _VarElement
import pyutilib.th as unittest
import pyutilib.services

class PyomoModel(unittest.TestCase):

    def setUp(self):
        self.model = AbstractModel()

    def construct(self,filename):
        self.instance = self.model.create(filename)


class TestSimpleObj(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)

    def tearDown(self):
        pass

    def test_expr1_option(self):
        """Test expr option"""
        model = ConcreteModel()
        model.x = Var(RangeSet(1,4),initialize=2)
        ans=0
        for i in model.x.keys():
            ans = ans + model.x[i]
        model.obj = Objective(expr=ans)
        instance = model.create()
        self.assertEqual(instance.obj(), 8)
        self.assertEqual(value(instance.obj), 8)

    def test_expr2_option(self):
        """Test expr option"""
        model = ConcreteModel()
        model.x = Var(initialize=2)
        model.obj = Objective(expr=model.x)
        instance = model.create()
        instance.x.reset()
        #print 'X',type(instance.obj.rule)
        self.assertEqual(instance.obj(), 2)
        self.assertEqual(value(instance.obj), 2)

    def test_rule_option(self):
        """Test rule option"""
        def f(model):
          ans=0
          for i in model.x.keys():
            ans = ans + model.x[i]
          return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.obj = Objective(rule=f)
        self.instance = self.model.create()
        self.assertEqual(self.instance.obj(), 8)
        self.assertEqual(value(self.instance.obj), 8)

    def test_arguments(self):
        """Test that arguments notare of type _SetContainer"""
        def rule(model):
            return 1
        try:
            self.model.obj = Objective(self.model, rule=rule)
        except TypeError:
            pass
        else:
            self.fail("Objective should only accept '_SetContainter's")
        

    def test_sense_option(self):
        """Test sense option"""
        def rule(model):
            return 1.0
        self.model.obj = Objective(sense=maximize, rule=rule)
        self.instance = self.model.create()
        self.assertEqual(self.instance.obj.sense, maximize)

    def test_dim(self):
        """Test dim method"""
        def rule(model):
            return 1
        self.model.obj = Objective(rule=rule)
        self.instance = self.model.create()
        self.assertEqual(self.instance.obj.dim(),0)

    def test_keys(self):
        """Test keys method"""
        def rule(model):
            return 1
        self.model.obj = Objective(rule=rule)
        self.instance = self.model.create()
        self.assertEqual(self.instance.obj.keys(),[None])

    def test_len(self):
        """Test len method"""
        def rule(model):
            return 1.0
        self.model.obj = Objective(rule=rule)
        self.instance = self.model.create()
        self.assertEqual(len(self.instance.obj),1)
        """Test rule option"""
        def f(model):
          ans=0
          for i in model.x.keys():
            ans = ans + model.x[i]
          return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.obj = Objective(rule=f)
        self.instance = self.model.create()
        self.assertEqual(len(self.instance.obj),1)


class TestArrayObj(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        self.model.A = Set(initialize=[1,2])

    def tearDown(self):
        pass

    def test_rule_option(self):
        """Test rule option"""
        def f(i,model):
          ans=0
          for j in model.x.keys():
            ans = ans + model.x[j]
          ans *= i
          return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.obj = Objective(self.model.A,rule=f)
        self.instance = self.model.create()
        self.assertEqual(self.instance.obj[1](), 8)
        self.assertEqual(self.instance.obj[2](), 16)
        self.assertEqual(value(self.instance.obj[1]), 8)
        self.assertEqual(value(self.instance.obj[2]), 16)

    def test_sense_option(self):
        """Test sense option"""
        self.model.obj = Objective(self.model.A,sense=maximize)
        self.instance = self.model.create()
        self.assertEqual(self.instance.obj.sense, maximize)

    def test_dim(self):
        """Test dim method"""
        self.model.obj = Objective(self.model.A)
        self.instance = self.model.create()
        self.assertEqual(self.instance.obj.dim(),1)

    def test_keys(self):
        """Test keys method"""
        def A_rule(i, model):
            return model.x
        self.model.x = Var()
        self.model.obj = Objective(self.model.A, rule=A_rule)
        self.instance = self.model.create()
        self.assertEqual(len(self.instance.obj.keys()),2)

    def test_len(self):
        """Test len method"""
        self.model.obj = Objective(self.model.A)
        self.instance = self.model.create()
        self.assertEqual(len(self.instance.obj),0)
        """Test rule option"""
        def f(model):
          ans=0
          for i in model.x.keys():
            ans = ans + model.x[i]
          return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.obj = Objective(rule=f)
        self.instance = self.model.create()
        self.assertEqual(len(self.instance.obj),1)


class Test2DArrayObj(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        self.model.A = Set(initialize=[1,2])

    def tearDown(self):
        pass

    def test_rule_option(self):
        """Test rule option"""
        def f(i,k,model):
          ans=0
          for j in model.x.keys():
            ans = ans + model.x[j]
          ans *= i
          return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.obj = Objective(self.model.A,self.model.A, rule=f)
        self.instance = self.model.create()
        try:
            self.assertEqual(self.instance.obj(),None)
            self.fail("Expected ValueError")
        except ValueError:
            pass
        self.instance.x.reset()
        self.assertEqual(self.instance.obj[1,1](), 8)
        self.assertEqual(self.instance.obj[2,1](), 16)
        self.assertEqual(value(self.instance.obj[1,1]), 8)
        self.assertEqual(value(self.instance.obj[2,1]), 16)

    def test_sense_option(self):
        """Test sense option"""
        self.model.obj = Objective(self.model.A,self.model.A,sense=maximize)
        self.instance = self.model.create()
        self.assertEqual(self.instance.obj.sense, maximize)

    def test_dim(self):
        """Test dim method"""
        self.model.obj = Objective(self.model.A,self.model.A)
        self.instance = self.model.create()
        self.assertEqual(self.instance.obj.dim(),2)

    def test_keys(self):
        """Test keys method"""
        def A_rule(i,j,model):
            return model.x
        self.model.x = Var()
        self.model.obj = Objective(self.model.A,self.model.A, rule=A_rule)
        self.instance = self.model.create()
        self.assertEqual(len(self.instance.obj.keys()),4)

    def test_len(self):
        """Test len method"""
        self.model.obj = Objective(self.model.A,self.model.A)
        self.instance = self.model.create()
        self.assertEqual(len(self.instance.obj),0)
        """Test rule option"""
        def f(model):
          ans=0
          for i in model.x.keys():
            ans = ans + model.x[i]
          return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.obj = Objective(rule=f)
        self.instance = self.model.create()
        self.assertEqual(len(self.instance.obj),1)


class MiscObjTests(pyutilib.th.TestCase):

    def test_constructor(self):
        a = Objective(name="b")
        self.assertEqual(a.name,"b")
        try:
            a = Objective(foo="bar")
            self.fail("Can't specify an unexpected constructor option")
        except ValueError:
            pass

    def test_contains(self):
        model=AbstractModel()
        model.a=Set(initialize=[1,2,3])
        model.x=Var()
        def b_rule(i,model):
            return model.x
        model.b=Objective(model.a, rule=b_rule)
        instance=model.create()
        self.assertEqual(2 in instance.b,True)
        tmp=[]
        for i in instance.b:
          tmp.append(i)
        self.assertEqual(len(tmp),3)

    def test_set_get(self):
        a = Objective()
        #try:
            #a.value = 1
            #self.fail("Can't set value attribute")
        #except AttributeError:
            #pass
        self.assertEqual(a(),None)
        #
        model=AbstractModel()
        model.x = Var(initialize=1)
        model.y = Var(initialize=2)
        model.obj = Objective()
        model.obj._data[None].expr = model.x+model.y
        instance = model.create()
        instance.reset()
        self.assertEqual(instance.obj(),3)

    def test_rule(self):
        def rule1(model):
            return None
        model=AbstractModel()
        model.o = Objective(rule=rule1)
        try:
            instance=model.create()
            self.fail("Error generating objective")
        except Exception:
            pass
        #
        def rule1(model):
            return 1.1
        model=AbstractModel()
        model.o = Objective(rule=rule1)
        instance=model.create()
        self.assertEqual(instance.o(),1.1)
        #
        def rule1(i,model):
            return 1.1
        model=AbstractModel()
        model.a=Set(initialize=[1,2,3])
        model.o = Objective(model.a,rule=rule1)
        instance=model.create()
        try:
            instance=model.create()
        except Exception:
            self.fail("Error generating objective")
        

if __name__ == "__main__":
   unittest.main()




#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the FAST README.txt file.
#  _________________________________________________________________________

import pyutilib.component.core
from coopr.pyomo.base import IPyomoPresolver, IPyomoPresolveAction


class NumericVarLabelsPresolver(pyutilib.component.core.SingletonPlugin):
    """
    This plugin changes variables names to use a simple name that
    uses the variable id.
    """

    pyutilib.component.core.implements(IPyomoPresolveAction)

    def __init__(self, **kwds):
        kwds['name'] = "numeric_varlabels"
        pyutilib.component.core.Plugin.__init__(self, **kwds)

    def rank(self):
        return 1000

    def preprocess(self,model):
        """
        The main routine to perform the preprocess
        """
        for id in model._var:
            model._var[id].label = "x_"+str(id)
            model._name_varmap[model._var[id].label] = model._var[id]
        for id in model._con:
            model._con[id].label = "c_"+str(id)
        return model




#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['BuildAction']

import logging

from coopr.pyomo.base.numvalue import *
from coopr.pyomo.base.indexed_component import IndexedComponent
import types
from pyutilib.component.core import alias

logger = logging.getLogger('coopr.pyomo')


class BuildAction(IndexedComponent):
    """A build action, which executes a rule for all valid indices"""

    """ Constructor
        Arguments:
           name         The name of this action
           index        The index set that defines the distinct parameters.
                          By default, this is None, indicating that there
                          is a single action.
           rule         The rule that is executed for every indice.
    """

    alias("BuildAction", "This component is used to inject arbitrary actions into the model construction process.  The action rule is applied to every index value.")

    def __init__(self, *args, **kwd):
        tkwd = {'ctype':BuildAction}
        IndexedComponent.__init__(self, *args, **tkwd)
        tmpname="unknown"
        self.domain=None
        self._rule=None
        for key in kwd.keys():
          if key == "name":
             tmpname=kwd[key]
          elif key == "rule":
             self.__dict__["_"+key] = kwd[key]
          else:
             raise ValueError, "BuildAction constructor: unknown keyword - "+key
        if not type(self._rule) is types.FunctionType:
            raise ValueEror, "BuildAction must have an 'rule' option specified whose value is a function"

    def pprint(self, ostream=None):
        if ostream is None:
           ostream = sys.stdout
        print >>ostream, "  ",self.name,":",

    def construct(self, data=None):
        """ Apply the rule to construct values in this set """
        if __debug__:
            if logger.isEnabledFor(logging.DEBUG):
                logger.debug("Constructing Action, name="+self._name+", from data="+`data`)
        if self._constructed:
            return
        self._constructed=True
        #
        # Update the index set dimension
        #
        self._compute_dim()
        for val in self._index:
              if isinstance(val,tuple):
                 tmp = list(val)
              elif val is None:
                 tmp = []
              else:
                 tmp = [val]
              tmp.append(self.model)
              tmp = tuple(tmp)
              self._rule(*tmp)





#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['BuildCheck']

import logging

from pyutilib.component.core import alias
from coopr.pyomo.base.numvalue import *
from coopr.pyomo.base.indexed_component import IndexedComponent
import types

logger = logging.getLogger('coopr.pyomo')


class BuildCheck(IndexedComponent):
    """A build check, which executes a rule for all valid indices"""

    """ Constructor
        Arguments:
           name         The name of this check
           index        The index set that defines the distinct parameters.
                          By default, this is None, indicating that there
                          is a single check.
           rule         The rule that is executed for every indice.
    """

    alias("BuildCheck", "This component is used to perform a test during the model construction process.  The action rule is applied to every index value, and if the function returns False an exception is raised.")

    def __init__(self, *args, **kwd):
        tkwd = {'ctype':BuildCheck}
        IndexedComponent.__init__(self, *args, **tkwd)
        tmpname="unknown"
        self.domain=None
        self._rule=None
        for key in kwd.keys():
          if key == "name":
             tmpname=kwd[key]
          elif key == "rule":
             self.__dict__["_"+key] = kwd[key]
          else:
             raise ValueError, "BuildCheck constructor: unknown keyword - "+key
        if not type(self._rule) is types.FunctionType:
            raise ValueEror, "BuildCheck must have an 'rule' option specified whose value is a function"

    def pprint(self, ostream=None):
        if ostream is None:
           ostream = sys.stdout
        print >>ostream, "  ",self.name,":",

    def construct(self, data=None):
        """ Apply the rule to construct values in this set """
        if __debug__:
            if logger.isEnabledFor(logging.DEBUG):
                logger.debug("Constructing Check, name="+self._name+", from data="+`data`)
        if self._constructed:
            return
        self._constructed=True
        #
        # Update the index set dimension
        #
        self._compute_dim()
        for val in self._index:
              if isinstance(val,tuple):
                 tmp = list(val)
              elif val is None:
                 tmp = []
              else:
                 tmp = [val]
              tmp.append(self.model)
              tmp = tuple(tmp)
              res = self._rule(*tmp)
              if not res:
                    raise ValueError, "BuildCheck %r identified error with index %r" % (self.name, tmp[:-1])





#
# Unit Tests for Utility Functions
#

import os
import sys
from os.path import abspath, dirname
sys.path.insert(0, dirname(dirname(abspath(__file__)))+"/../..")
currdir = dirname(abspath(__file__))+os.sep

import pyutilib.th as unittest
from coopr.pyomo import *
from nose.tools import nottest
import pickle


class TestNumericValue(unittest.TestCase):

    def test_expr1(self):
        model = AbstractModel()
        model.A = Set(initialize=[1,2,3])
        model.B = Param(model.A,initialize={1:100,2:200,3:300})
        model.x = Var(model.A)
        model.y = Var(model.A)
        instance=model.create()
        expr = dot_product(instance.x,instance.B,instance.y)
        OUTPUT=open(currdir+"test_expr1.out","w")
        expr.pprint(ostream=OUTPUT)
        OUTPUT.close()
        self.assertFileEqualsBaseline(currdir+"test_expr1.out",currdir+"test_expr1.txt")

    def test_expr2(self):
        model = AbstractModel()
        model.A = Set(initialize=[1,2,3])
        model.B = Param(model.A,initialize={1:100,2:200,3:300})
        model.x = Var(model.A)
        model.y = Var(model.A)
        instance=model.create()
        expr = dot_product(instance.x,instance.B,instance.y, index=[1,3])
        OUTPUT=open(currdir+"test_expr2.out","w")
        expr.pprint(ostream=OUTPUT)
        OUTPUT.close()
        self.assertFileEqualsBaseline(currdir+"test_expr2.out",currdir+"test_expr2.txt")

    def test_expr3(self):
        model = AbstractModel()
        model.A = Set(initialize=[1,2,3])
        model.B = Param(model.A,initialize={1:100,2:200,3:300})
        model.x = Var(model.A)
        model.y = Var(model.A)
        instance=model.create()
        expr = dot_product(instance.x,instance.B,denom=instance.y, index=[1,3])
        OUTPUT=open(currdir+"test_expr3.out","w")
        expr.pprint(ostream=OUTPUT)
        OUTPUT.close()
        self.assertFileEqualsBaseline(currdir+"test_expr3.out",currdir+"test_expr3.txt")

    def test_expr4(self):
        model = AbstractModel()
        model.A = Set(initialize=[1,2,3])
        model.B = Param(model.A,initialize={1:100,2:200,3:300})
        model.x = Var(model.A)
        model.y = Var(model.A)
        instance=model.create()
        expr = dot_product(denom=[instance.y,instance.x])
        OUTPUT=open(currdir+"test_expr4.out","w")
        expr.pprint(ostream=OUTPUT)
        OUTPUT.close()
        self.assertFileEqualsBaseline(currdir+"test_expr4.out",currdir+"test_expr4.txt")

class TestPickle(pyutilib.th.TestCase):

    def test_p1(self):
        model = AbstractModel()
        model.A = Set(initialize=[1,2,3])
        model.B = Param(model.A,initialize={1:100,2:200,3:300})
        model.x = Var(model.A)
        model.y = Var(model.A)
        str = pickle.dumps(model)
        tmodel = pickle.loads(str)
        instance=tmodel.create()
        expr = dot_product(instance.x,instance.B,instance.y)
        OUTPUT=open(currdir+"test_expr1.out","w")
        expr.pprint(ostream=OUTPUT)
        OUTPUT.close()
        self.assertFileEqualsBaseline(currdir+"test_expr1.out",currdir+"test_expr1.txt")

    def test_p2(self):
        model = AbstractModel()
        model.A = Set(initialize=[1,2,3])
        model.B = Param(model.A,initialize={1:100,2:200,3:300})
        model.x = Var(model.A)
        model.y = Var(model.A)
        tmp=model.create()
        str = pickle.dumps(tmp)
        instance = pickle.loads(str)
        expr = dot_product(instance.x,instance.B,instance.y)
        OUTPUT=open(currdir+"test_expr1.out","w")
        expr.pprint(ostream=OUTPUT)
        OUTPUT.close()
        self.assertFileEqualsBaseline(currdir+"test_expr1.out",currdir+"test_expr1.txt")

    @unittest.skipIf(sys.version_info[:2] < (2,6), "Skipping test because the sparse_dict repn is not supported")
    def test_p3(self):
        def rule1(model):
            return (1,model.x+model.y[1],2)
        def rule2(i,model):
            return (1,model.x+model.y[1]+i,2)

        model = AbstractModel()
        model.a = Set(initialize=[1,2,3])
        model.A = Param(initialize=1)
        model.B = Param(model.a, repn='sparse_dict')
        model.x = Var(initialize=1,within=Reals)
        model.y = Var(model.a, initialize=1,within=Reals)
        model.obj = Objective(rule=lambda model: model.x+model.y[1])
        model.obj2 = Objective(model.a,rule=lambda i,model: i+model.x+model.y[1])
        model.con = Constraint(rule=rule1)
        model.con2 = Constraint(model.a, rule=rule2)
        instance = model.create()
        try:
            str = pickle.dumps(instance)
            self.fail("Expected pickling error due to the use of lambda expressions")
        except pickle.PicklingError:
            pass
        except TypeError:
            pass



if __name__ == "__main__":
   unittest.main()



# an example of an indexed piecewise function, i.e., a set of piecewise functions, each identified by a unique index.

from coopr.pyomo import *

def compute_breakpoints(i, m):
  return [0]

def compute_slopes(i, m):
  return [-1, 1]

model = AbstractModel()

model.Index = Set(initialize=[1, 2])

model.X = Var(model.Index)

model.Z = Piecewise(model.Index,
                    breakpoint_rule=compute_breakpoints,
                    slope_rule=compute_slopes,
                    offset=2,
                    value=model.X)

def objective_rule(model):
   return sum([model.Z[i] for i in model.Index])

model.obj = Objective(rule = objective_rule, sense=minimize)



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['ModelData']

import logging
from plugin import *

logger = logging.getLogger('coopr.pyomo')


class ModelData(object):
    """
    An object that manages data for a model.

    This object contains the interface routines for importing and
    exporting data from external data sources.
    """

    def __init__(self, **kwds):
        """
        Constructor
        """
        #
        # This is the data that is imported from various data sources
        #
        self._default={}
        # _data[scenario][symbol] -> data
        self._data={}
        #
        self._info=[]
        self._model = kwds.pop('model', None)

        if 'filename' in kwds:
            if not kwds['filename'] is None:
                filename = kwds['filename']
                del kwds['filename']
                self.add(filename, **kwds)
                self.read()
        elif len(kwds) > 0:
            raise ValueError, "Unknown options: %s" % str(kwds.keys())

    def initialize(self, component, data, namespace=None):
        self._data[namespace][component] = data

    def add(self, filename, **kwds):
        tmp = filename.split(".")[-1]
        data = DataManagerFactory(tmp)
        data.initialize(filename, **kwds)
        self._info.append(data)

    def read(self, model=None):
        """
        Read data
        """
        if model is not None:
           self._model=model
        if self._model is None:
           raise ValueError, "Cannot read model data without a model"
        #
        # Although we want to load data in a particular order, it
        # may make sense to open/close all data resources all at once.
        # Should we do this by default, or make this a configuration option
        # for this class?
        #
        for data in self._info:
            if __debug__:
                logger.debug("Importing data...")
            data.open()
            if not data.read():
                print "Warning: error occured while processing %s" % str(data)
            data.close()
            if __debug__:
                logger.debug("Processing data ...")
            #
            # Update the _data and _default dictionaries with data that
            # was read in.
            #
            status = data.process(self._model, self._data, self._default)
            data.clear()
            if __debug__:
                logger.debug("Done.")




__all__ = ['Objective']

import sys
import logging

from expr import *
from indexed_component import IndexedComponent
from numtypes import *
from numvalue import *
from numvalue import create_name
from label import *
from pyutilib.component.core import alias
from sets import _BaseSet, _SetContainer, _ProductSet
from var import Var, _VarValue, _VarArray, _VarElement
import pyutilib.math
import pyutilib.misc

from component import Component
from set_types import *
from sets import _SetContainer

logger = logging.getLogger('coopr.pyomo')


class ObjectiveData(NumericValue):

    __slots__ = ['obj', 'expr', 'label', 'id', 'active', 'repn', 'ampl_repn']

    def __init__(self, expr=None, name=None, obj=None):
        NumericValue.__init__(self,name,Reals,None,True)

        self.expr = expr
        self.obj = obj

        # the label is a modification of the input name, with certain symbols
        # that are typically problematic for solver input files re-mapped (e.g., 
        # curly braces) to more friendly symbols. create a default label right
        # away - it can be modified if a particular solver needs it.
        # self.label = None (is set one way or another via the following method)
        self.sync_label()

        self.id = -1

        self.active = True

        self.repn = None
        self.ampl_repn = None

    def __getstate__(self):
       result = NumericValue.__getstate__(self)
       for i in ObjectiveData.__slots__:
          result[i] = getattr(self, i)
       return result        

    def sync_label(self):
        if self.name is not None:
            self.label = label_from_name(self.name)
            if (self.obj is not None) and (self.obj.model is not None):
                self.obj.model._name_objmap[self.label] = self
        else:
            self.label = None

    def __call__(self, exception=True):
        if self.expr is None:
            return None
        return self.expr()

    def polynomial_degree(self):
        if self.expr is None:
            return None
        return self.expr.polynomial_degree()

    def activate(self):
        self.active=True

    def deactivate(self):
        self.active=False


class Objective(IndexedComponent, NumericValue):
    """An object that defines a objective expression"""

    alias('Objective', 'Expressions that are minimized or maximized in a '    \
          'model.')

    def __init__(self, *args, **kwargs):
        """Construct an objective expression with rule to construct the
           expression
        """
        tkwargs = {'ctype':Objective}
        IndexedComponent.__init__(self, *args, **tkwargs)

        self._data = {}
        if args == ():
            self._data[None] = ObjectiveData(obj=self)
        else:
            for i in args:
                if not ( isinstance(i, _SetContainer) ):
                    msg  = 'Invalid argument.  Objective expects arguments of '
                    msg += "type 'Set', not type '%s'"
                    raise TypeError, msg % str( type(i).__name__ )

        # this is a hack for now; we eventually want a
        # "QuadraticObjective" class.
        self._quad_subexpr = None

        tmpname  = kwargs.pop('name', 'unknown')
        tmpsense = kwargs.pop('sense', minimize )
        tmprule  = kwargs.pop('rule', None )
        tmprule  = kwargs.pop('expr', tmprule )
        self.doc = kwargs.pop('doc', None )
        self._no_rule_init = kwargs.get('noruleinit', None )

        # _no_rule_init is a flag specified by the user to indicate that no
        # construction rule will be specified, and that constraints will be
        # explicitly added by the user. set via the "noruleinit" keyword. value
        # doesn't matter, as long as it isn't "None".

        if ( kwargs ): # if dict not empty, there's an error.  Let user know.
            msg = "Creating constraint '%s': unknown option(s)\n\t%s"
            msg = msg % ( tmpname, ', '.join(kwargs.keys()) )
            raise ValueError, msg

        NumericValue.__init__(self,tmpname,None,None,True)        

        if None in self._data:
            self._data[None].name = self.name
            self._data[None].sync_label()

        self.sense   = tmpsense
        self.rule    = tmprule
        self.trivial = False # True if all objective indicies have trivial expressions.
        self._constructed = False

    def __getstate__(self):
       result = NumericValue.__getstate__(self)
       for key,value in self.__dict__.iteritems():
          result[key]=value
       return result                    

    def clear(self):
        self._data = {}

    def __call__(self, exception=True):
        if len(self._data) == 0:
            return None
        if None in self._data:
            if self._data[None].expr is None:
                return None
            return self._data[None].expr()
        if exception:
            msg = 'Cannot compute the value of an array of objectives'
            raise ValueError, msg

    def dim(self):
        return self._ndim

    def __len__(self):
        return len(self._data.keys())

    def keys(self):
        return self._data.keys()
        #return self._index

    def __contains__(self,ndx):
        return ndx in self._data
        #return ndx in self._index

    def __getitem__(self,ndx):
        """This method returns a ObjectiveData object.  This object can be
           coerced to a numeric value using the value() function, or using
           explicity coercion with float().
        """
        if ndx in self._data:
           return self._data[ndx]
        msg = "Unknown index in objective '%s': %s"
        raise KeyError, msg % ( self.name, str(ndx) )

    def __iter__(self):
        return self._data.keys().__iter__()
        #return self._index.__iter__()

    def _apply_rule(self,arg,ndx):
        tmp = self.rule(*arg)
        if tmp is None:
           return None
        if type(tmp) in [bool,int,long,float]:
           return NumericConstant(None,None,tmp)
        return tmp


    def is_minimizing ( self ):
        return minimize == self.sense


    def construct(self, data=None):
        if __debug__:
            logger.debug("Constructing objective %s", self.name)
        if (self._no_rule_init is not None) and (self.rule is not None):
            msg = 'WARNING: noruleinit keyword is being used in conjunction ' \
                  "with rule keyword for objective '%s'; defaulting to "      \
                  'rule-based construction'
            print msg % self.name
        if self.rule is None:
           if self._no_rule_init is None:
               msg = 'WARNING: No construction rule or expression specified ' \
                     "for objective '%s'"
               print msg % self.name
           return
        if self._constructed:
            return
        self._constructed=True
        #
        if isinstance(self.rule,Expression) or isinstance(self.rule,_VarValue):
            if None in self._index or len(self._index) == 0 or self._index[None] is None:
                self._data[None].expr = self.rule
            else:
                msg = 'Cannot define multiple indices in an objective with a' \
                      'single expression'
                raise IndexError, msg
        #
        elif self.rule is not None:
           if self._ndim==0:
              tmp = self._apply_rule((self.model,),None)
              if tmp is not None and not (type(tmp) in (int,long,float)       \
                                          and tmp == 0):
                 self._data[None].expr = tmp
              self._data[None].label = label_from_name(self.name)
           else:
              _name=self.name
              for val in self._index:
                if type(val) is tuple:
                   tmp = list(val)
                else:
                   tmp = [val]
                tmp.append(self.model)
                tmp = tuple(tmp)
                tmp = self._apply_rule(tmp,val)
                if tmp is not None and not (type(tmp) in (int,long,float)     \
                                            and tmp == 0):
                    self._data[val] = ObjectiveData(expr=tmp,
                                                   name=create_name(_name,val), obj=self)
           #self._index = self._data.keys()
        #
        if None in self._data:
            self._data[None].label = label_from_name(self.name)
        #
        # update the parent model (if any) name->constraint map.
        #
        if self.model is not None:
           for objective_data in self._data.itervalues():
              self.model._name_objmap[objective_data.label] = objective_data

    def pprint(self, ostream=None):
        if ostream is None:
           ostream = sys.stdout
        print >>ostream, "  ",self.name,":",
        print >>ostream, "\tSize="+str(len(self._data.keys())),
        if isinstance(self._index,_BaseSet):
           print >>ostream, "\tIndex=",self._index.name
        else:
           print >>ostream,""
        for key in self._data:
          if not self._data[key].expr is None:
                print >>ostream, "\t",
                self._data[key].expr.pprint(ostream)
                print >>ostream, ""
        if self._quad_subexpr is not None:
           print >>ostream, "\tQuadratic sub-expression: ",
           self._quad_subexpr.pprint(ostream)
           print >>ostream, ""

    def display(self, prefix="", ostream=None):
        if ostream is None:
           ostream = sys.stdout
        print >>ostream, prefix+"Objective "+self.name,":",
        print >>ostream, "  Size="+str(len(self))
        if None in self._data:
            if self._data[None].expr is None:
                val = 'none'
            else:
                val = pyutilib.misc.format_io(self._data[None].expr(exception=False))
            print >>ostream, '%s  Value=%s' % (prefix, val)
        else:
           for key in self._data:
             if not self._data[key].active:
                continue
             val = self._data[key].expr(exception=False)
             print >>ostream, prefix+"  "+str(key)+" : "+str(val)



# identical to example1 - just more breakpoints.
#
# because the left-most breakpoint equals -2.5,
# the fixed "reference" point is (-2.5,0). the
# optimal variable value for this problem is
# obviously x=0. however, because the fixed
# point equals (-2.5,0) and the slope from x=-2.5
# x=0 is -1, the optimal objective function
# value equals -2.5.

from coopr.pyomo import *

model = AbstractModel()

model.X = Var(bounds=(-5,5))

breakpoints = [-2.5, 0, 2.5]
slopes =      [-10, -1, 1, 10]

model.Z = Piecewise(breakpoints=breakpoints,
                    slopes=slopes,
                    value=model.X)

model.obj = Objective(rule = lambda model: model.Z, sense=minimize)



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the FAST README.txt file.
#  _________________________________________________________________________

import pyutilib.misc
import pyutilib.component.core
from coopr.pyomo.base import IPyomoPresolver, IPyomoPresolveAction


class SimplePreprocessor(pyutilib.component.core.SingletonPlugin):
    """
    This plugin simply applies preprocess actions in a fixed order
    and then returns the modified model instance.
    """

    pyutilib.component.core.implements(IPyomoPresolver)

    def __init__(self, **kwds):
        kwds['name'] = "simple_preprocessor"
        pyutilib.component.core.Plugin.__init__(self, **kwds)
        self.actions = pyutilib.component.core.ExtensionPoint(IPyomoPresolveAction)
        self.active_actions = set()
        self.action_rank = {}
        self.initialized = False

    def _initialize(self):
        if self.initialized:
            return
        self.initialized = True
        for action in self.actions():
            self.activate_action(action.name)

    def get_actions(self):
        """Return a list of preprocess actions, with their ranks"""
        self._initialize()
        ans = []
        for action in self._order_actions():
            ans.append( (action, self.action_rank[action]) )
        return ans

    def activate_action(self, action, rank=None):
        """Activate an action"""
        self._initialize()
        tmp = self.actions.service(action)
        if tmp is None:
            raise ValueError, "Cannot activate unknown action %s" % action
        self.active_actions.add(action)
        if rank is None:
            rank = tmp.rank()
        self.action_rank[action] = rank

    def deactivate_action(self, action):
        """Deactivate an action"""
        self._initialize()
        if action in self.active_actions:
            self.active_actions.remove(action)
            del self.action_rank[action]

    def preprocess(self,model):
        """
        Apply the preprocess actions to this instance, and return the revised instance.
        """
        self._initialize()
        for action in self._order_actions():
            model = self.actions.service(action).preprocess(model)
        return model

    def _order_actions(self):
        actions = list(self.active_actions)
        ranks = []
        for item in actions:
            ranks.append(self.action_rank[item])
        index = pyutilib.misc.sort_index(ranks)
        sorted=[]
        for i in index:
            sorted.append( actions[i] )
        return sorted
            



#
# Unit Tests for BuildAction() Objects
#
# PyomoModel                Base test class
# Simple                    Test singleton parameter
# Array1                    Test arrays of parameters
#

import unittest
import os
import sys
from os.path import abspath, dirname
sys.path.insert(0, dirname(dirname(abspath(__file__)))+"/../..")
from coopr.pyomo import *

class PyomoModel(unittest.TestCase):

    def setUp(self):
        self.model = AbstractModel()

    def construct(self,filename):
        self.instance = self.model.create(filename)


def action1_fn(model):
    model.A = 4.3
    
def action2_fn(i,model):
    if i in model.A:
        model.A[i] = value(model.A[i])+i
    

class Simple(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        #
        # Create model instance
        #
        self.model.A = Param(initialize=3.3)
        self.model.action1 = BuildAction(rule=action1_fn)
        self.instance = self.model.create()

    def tearDown(self):
        if os.path.exists("param.dat"):
           os.remove("param.dat")

    def test_value(self):
        """Check the value of the parameter"""
        tmp = value(self.instance.A)
        self.assertEqual( type(tmp), float)
        self.assertEqual( tmp, 4.3 )

    def test_getattr(self):
        """Check the use of the __getattr__ method"""
        self.assertEqual( self.instance.A.value, 4.3)



class Array1(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        #
        # Create model instance
        #
        self.model.Z = Set(initialize=[1,3])
        self.model.A = Param(self.model.Z, initialize={1:1.3})
        self.model.action2 = BuildAction(self.model.Z, rule=action2_fn)
        self.instance = self.model.create()

    def test_value(self):
        """Check the value of the parameter"""
        tmp = value(self.instance.A[1])
        self.assertEqual( type(tmp), float)
        self.assertEqual( tmp, 2.3 )


class Array2(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        #
        # Create model instance
        #
        self.model.Z = Set(initialize=[1,3])
        self.model.A = Param(self.model.Z, initialize=1.3)
        self.model.action2 = BuildAction(self.model.Z, rule=action2_fn)
        self.instance = self.model.create()

    def test_getitem(self):
        """Check the use of getitem"""
        self.assertEqual( self.instance.A[1], 2.3)
        self.assertEqual( value(self.instance.A[3]), 4.3)


if __name__ == "__main__":
   unittest.main()



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['value', 'NumericValue', 'as_numeric', 'NumericConstant', 'ZeroConstant', 'OneConstant', 'is_constant']

import plugin
import pyutilib.math
import sys
from set_types import Reals, Any


##------------------------------------------------------------------------
##
## Standard types of expressions
##
##------------------------------------------------------------------------

def create_name(name, ndx):
    if ndx is None:
        return name
    if type(ndx) is tuple:
        tmp = str(ndx).replace(', ',',')
        return name+"["+tmp[1:-1]+"]"
    return name+"["+str(ndx)+"]"

def value(obj):
    """
    A utility function that returns the value of a Pyomo object or expression.

    If the argument is None, a numeric value or a string, then this function simply
    returns the argument.  Otherwise, if the argument is a NumericValue then the
    __call__ method is executed.
    """
    if obj is None:
        return None
    if type(obj) in (bool,int,long,float,str):
        return obj
    if not isinstance(obj, NumericValue):
        raise ValueError, "Object "+str(obj)+" is not a NumericValue object"
    tmp = obj()
    if tmp is None:
        raise ValueError, "No value for uninitialized NumericValue object "+obj.name
    return tmp

def is_constant(obj):
    """
    A utility function that returns a boolean that indicates whether the object is a constant
    """
    if obj.__class__ is int or obj.__class__ is float:
        return True
    try:
        return obj.is_constant()
    except AttributeError:
        if obj.__class__ is (obj + 0).__class__ or \
                obj.__class__ is bool:
            return True
    except:
        pass
    raise ValueError, "This object is not a numeric value: "+str(obj)

def as_numeric(obj):
    """Verify that this obj is a NumericValue or intrinsic value"""
    if obj.__class__ is int or obj.__class__ is float:
        return NumericConstant(None,None,obj)
    try:
        return obj.as_numeric()
    except AttributeError:
        pass
    try:
        if obj.__class__ is (obj + 0).__class__ or \
                obj.__class__ is bool:
            return NumericConstant(None, None, obj)
    except:
        pass
    raise ValueError, "This object is not a numeric value: "+str(obj)


class NumericValue(object):
    """An object that contains a numeric value"""

    __slots__ = ['name','domain','value']

    __hash__ = None

    #
    # the validate_value argument only applies to the value supplied in the constructor, and does not prevent 
    # subsequent value assignments from being checked.
    #

    # IMPT: If a specific argument is intended to be left unassigned, use the following default values:
    # name: None
    # domain: None
    # value: None
    # validate_value: True

    def __init__(self, name, domain, value, validate_value):

        self.name = name
        self.domain = domain

        if (value and pyutilib.math.is_nan(value)):
            value = pyutilib.math.nan

        # a numeric value is only constrained by the domain,
        # if specified. avoid invoking set_value() here to
        # accelerate object construction, which takes
        # significant time due to the typical number of
        # occurrences. similarly, we are avoiding the
        # invocation of _valid_value. 
        if (value is not None) and (validate_value is True) and (self.domain is not None) and (value not in self.domain):
           raise ValueError, "Numeric value `"+str(value)+"` is not in domain "+str(self.domain)            

        self.value = value

    # defining slots requires the definition of __getstate__ and __setstate__ for pickle'ing.
    # each derived class needs to over-ride this method to both call the base class method
    # (which will populate the dictionary with slots specific to the base class, and to add
    # any slots to the dictionary unique to the derived class.
    def __getstate__(self):
       return dict((i, getattr(self, i)) for i in NumericValue.__slots__)
    
    def __setstate__(self, data_dict):
       for (slot_name, value) in data_dict.iteritems():
          setattr(self, slot_name, value)
        
    def is_constant(self):
        return True

    def is_expression(self):
        return False

    def is_relational(self):
        return False

    def is_indexed(self):
        return False

    def polynomial_degree(self):
        return 0

    def fixed_value(self):
        return False

    def as_numeric(self):
        return self

    def pprint(self, ostream=None):
        raise IOError, "NumericValue:pprint is not defined"     #pragma:nocover

    def display(self, ostream=None):
        self.pprint(ostream=ostream)

    def reset(self):            #pragma:nocover
        pass                    #pragma:nocover

    def set_value(self, val):
        if self._valid_value(val):
            self.value=val

    def __nonzero__(self):
        if self.value is None:
            raise ValueError, "Numeric value is undefined"
        if self.value:
            return True
        return False

    def __call__(self, exception=True):
        return self.value

    def __float__(self):
        tmp = self.__call__()
        if tmp is None:
            raise ValueError, "Cannot coerce numeric value `"+self.name+"' to float because it is uninitialized."
        return float(tmp)

    def __int__(self):
        tmp = self.__call__()
        if tmp is None:
            raise ValueError, "Cannot coerce numeric value `"+self.name+"' to integer because it is uninitialized."
        return int(tmp)

    def _valid_value(self,value,use_exception=True):
        #print "HERE X", self.domain is None
        ans = value is None or self.domain is None or value in self.domain
        if not ans and use_exception:
           raise ValueError, "Numeric value `"+str(value)+"` is not in domain "+str(self.domain)
        return ans

    def X__getattr__(self,name):
        #print "GETATTR",name
        #d = self.__dict__
        try:
            return self.__dict__[name]
        except:
            pass
        try:
            return self.__dict__["_attr_"+name]
        except:
            pass
        raise AttributeError, "Unknown attribute `"+str(name)+"' for object with type "+str(type(self))

    def X__setattr__(self,name,val):
        ##print "SETATTR",name,val
        if name == "__class__":
           self.__class__ = val
           return
        if name[0] == "_":
           self.__dict__[name] = val
           return
        if name in self._standard_attr:
           self.__dict__["_attr_"+name] = val
           return
        raise AttributeError, "Unallowable attribute `"+name+"`"
        #self._standard_attr[name] = val

    def __lt__(self,other):
        """Less than operator

        (Called in response to 'self < other' or 'other > self'.)
        """
        return expr.generate_relational_expression('<', self, as_numeric(other))

    def __gt__(self,other):
        """Greater than operator

        (Called in response to 'self > other' or 'other < self'.)
        """
        return expr.generate_relational_expression('<', as_numeric(other), self)

    def __le__(self,other):
        """Less than or equal operator

        (Called in response to 'self <= other' or 'other >= self'.)
        """
        return expr.generate_relational_expression('<=', self, as_numeric(other))

    def __ge__(self,other):
        """Greater than or equal operator

        (Called in response to 'self >= other' or 'other <= self'.)
        """
        return expr.generate_relational_expression('<=', as_numeric(other), self)

    def __eq__(self,other):
        """Equal to operator

        (Called in response to 'self = other'.)
        """
        return expr.generate_relational_expression('==', self, as_numeric(other))

    def __add__(self,other):
        """Binary addition

        (Called in response to 'self + other'.)
        """
        return expr.generate_expression('add',self,other)

    def __sub__(self,other):
        """ Binary subtraction

        (Called in response to 'self - other'.)
        """
        return expr.generate_expression('sub',self,other)

    def __mul__(self,other):
        """ Binary multiplication

        (Called in response to 'self * other'.)
        """
        return expr.generate_expression('mul',self,other)

    def __div__(self,other):
        """ Binary division

        (Called in response to 'self / other'.)
        """
        return expr.generate_expression('div',self,other)

    def __pow__(self,other):
        """ Binary power

        (Called in response to 'self ** other'.)
        """
        return expr.generate_expression('pow',self,other)

    def __radd__(self,other):
        """Binary addition

        (Called in response to 'other + self'.)
        """
        return expr.generate_expression('radd',self,other)

    def __rsub__(self,other):
        """ Binary subtraction

        (Called in response to 'other - self'.)
        """
        return expr.generate_expression('rsub',self,other)

    def __rmul__(self,other):
        """ Binary multiplication

        (Called in response to 'other * self'.)
        """
        return expr.generate_expression('rmul',self,other)
        
    def __rdiv__(self,other):
        """ Binary division

        (Called in response to 'other / self'.)
        """
        return expr.generate_expression('rdiv',self,other)

    def __rpow__(self,other):
        """ Binary power

        (Called in response to 'other ** self'.)
        """
        return expr.generate_expression('rpow',self,other)

    def __iadd__(self,other):
        """Binary addition

        (Called in response to 'self += other'.)
        """
        return expr.generate_expression('iadd',self,other)

    def __isub__(self,other):
        """ Binary subtraction

        (Called in response to 'self -= other'.)
        """
        return expr.generate_expression('isub',self,other)

    def __imul__(self,other):
        """ Binary multiplication

        (Called in response to 'self *= other'.)
        """
        return expr.generate_expression('imul',self,other)
        
    def __idiv__(self,other):
        """ Binary division

        (Called in response to 'self /= other'.)
        """
        return expr.generate_expression('idiv',self,other)

    def __ipow__(self,other):
        """ Binary power

        (Called in response to 'self **= other'.)
        """
        return expr.generate_expression('ipow',self,other)

    def __neg__(self):
        """ Negation

        (Called in response to '- self'.)
        """
        return expr.generate_expression('neg',self)

    def __pos__(self):
        """ Positive expression

        (Called in response to '+ self'.)
        """
        return self

    def __abs__(self):
        """ Absolute value

        (Called in response to 'abs(self)'.)
        """
        return expr.generate_expression('abs',self)


class NumericConstant(NumericValue):
    """An object that contains a constant numeric value"""

    __slots__ = [] # adds no attributes above and beyond those defined in the base clss.

    # IMPT: If a specific argument is intended to be left unassigned, use the following default values:
    # name: None
    # domain: None
    # value: None

    def __init__(self, name, domain, value):
        if domain is None:
           domain = Reals
        # NOTE: we in general don't perform domain validation of numeric constants,
        #       especially because the user/caller has specified the value, and 
        #       typically does not specify the domain. The domain might be useful 
        #       with meta-level symbolic expression analysis, but for now we treat this as
        #       a Any.
        NumericValue.__init__(self, name, domain, value, False)

    def fixed_value(self):
        return True

    def __call__(self, exception=True):
        return self.value

    def __str__(self):
        return str(self.value)

    def pprint(self, ostream=None):
        if ostream is None:
           ostream = sys.stdout
        print >>ostream, str(self),


ZeroConstant = NumericConstant(None, None, 0)
OneConstant = NumericConstant(None, None, 1)

import expr
